@@ -1,40 +1,99 @@
-# Copyright (c) 2002-2009 Infrae. All rights reserved.
+# -*- coding: utf-8 -*-
+# Copyright (c) 2002-2013 Infrae. All rights reserved.
 # See also LICENSE.txt
-# $Id$
 
 
 from zope import interface
+from zope.container.interfaces import INameChooser
 from zope.annotation import IAttributeAnnotatable
 from grokcore.component.interfaces import IContext
 
 
+class IHTTPHeadersSettings(interface.Interface):
+    """Settings for IHTTPResponseHeaders on Silva objects
+    """
+    # XXX Add an override for the developer to always disable caching without
+    # letting the user choose.
+    http_disable_cache = interface.Attribute('Prevent caching altogether')
+    http_max_age = interface.Attribute('Max caching ime')
+    http_last_modified = interface.Attribute('Add Last-Modified headers')
+
+
 class ICustomizable(interface.Interface):
-    """Customizable content
+    """Layout-customizable content
+    """
+
+
+class ISilvaXMLExportable(interface.Interface):
+    """Exportable content in Silva XML
+    """
+
+
+class IXMLZEXPExportable(ISilvaXMLExportable):
+    """Content that is exportable via the fallback ZEXP exportable
+    handler.
     """
 
 
 class IDirectlyRendered(interface.Interface):
-    """Content directly rendered without the help of a layout.
+    """Content directly rendered without the help of a layout
+    """
+
+
+class IQuotaObject(interface.Interface):
+    """Content that can be accounted inside a site quota
+    """
+
+    def get_quota_usage():
+        """Return the space used by this content.
+        """
+
+    def update_quota():
+        """Update used space, and verify quota for this content.
+        """
+
+    def reset_quota():
+        """Reset the status of the quota.
+        """
+
+class IQuotaContainer(interface.Interface):
+    """Container that aggregate the usage of the contained quota items
+    """
+    used_space = interface.Attribute(u"Used space by quota items.")
+
+    def update_used_space(delta, verify):
+        """Update used space with ``delta``, and verify quota for this
+        folder. This is a private method (meaning it can't be called
+        from the ZODB).
+        """
+
+
+class IReferable(interface.Interface):
+    """Content can be referred by an another
     """
 
 
 class ISecurity(interface.Interface):
-    """Content object with author and creator information.
+    """Content with author and creator information
     """
 
-    def sec_get_last_author_info():
-        """Return the member object corresponding to the last author
-        who modified this content object.
-        """
-
-    def sec_get_creator_info():
+    def get_creator_info():
         """Return the member object corresponding to the creator of
         this content object.
         """
 
+    def get_last_author_info():
+        """Return the member object corresponding to the last author
+        who modified this content object.
+        """
+
+    def set_last_author_info(user):
+        """Set the last author information to the given user.
+        """
+
 
 class ITitledObject(interface.Interface):
-    """Content objects with a title.
+    """Content with a title
     """
 
     def get_title():
@@ -42,33 +101,80 @@ class ITitledObject(interface.Interface):
         """
 
     def get_title_or_id():
-        """The title or id of the content object.
+        """The title, or the identifier if there is no title of the
+        content object.
+        """
+
+    def get_short_title():
+        """Return the short title of the content object. If there is
+        no short title, when the title is returned. If there is no
+        title, when the identifier is returned.
         """
 
     def set_title(title):
-        """Change the title of the content object.
+        """Change the title of the content object if possible.
         """
 
 
-class ISilvaObject(IContext, IAttributeAnnotatable, ISecurity, ITitledObject, ICustomizable):
-    """All Silva content object.
+class IEditableTitledObject(ITitledObject):
+
+    def get_editable():
+        """Get the editable version (may be object itself). Returns
+        ``None`` if there is no such version.
+        """
+
+    def get_title_editable():
+        """Get the title of the editable version if possible.
+        """
+
+    def get_short_title_editable():
+        """Get the short title of the editable version if possible.
+        """
+
+    def get_title_or_id_editable():
+        """Get the title of the editable version if possible, or id if
+        not available.
+        """
+
+
+class ISilvaObject(IContext,
+                   IReferable,
+                   IAttributeAnnotatable,
+                   ISecurity,
+                   IEditableTitledObject,
+                   ISilvaXMLExportable):
+    """Silva Content
     """
 
+    def get_silva_object():
+        """Used by acquisition to get the nearest containing
+        SilvaObject.
+        """
+
     def get_creation_datetime():
         """Return of creation datetime of the object. Return None if
         not supported.
         """
 
     def get_modification_datetime():
-        """Return the last modification datetime of the object. Return None if not
-        supported.
+        """Return the last modification datetime of the object. Return
+        None if not supported.
         """
 
-    def get_editable():
-        """Get the editable version (may be object itself if no versioning).
-        Returns None if there is no such version.
+    def is_deletable():
+        """Returns True if object is deletable right now.
         """
 
+
+class IImageIncluable(ISilvaObject):
+    """Content represent an image that can be included in text
+    """
+
+
+class IViewableObject(ICustomizable):
+    """Silva Content viewable by the public
+    """
+
     def get_previewable():
         """Get the previewable version (may be the object itself if no
         versioning).
@@ -81,33 +187,18 @@ class ISilvaObject(IContext, IAttributeAnnotatable, ISecurity, ITitledObject, IC
         Returns None if there is no such version.
         """
 
-    def preview():
-        """Render this object using the public view defined in the
-        view registry.
 
-        This should use methods on the object itself and the version object
-        obtained by ``get_previewable()`` to render the object to HTML.
-        """
+class IPublishable(ISilvaObject, IViewableObject):
+    """Content that can be published to the public
 
-    def view():
-        """Render this object using the public view defined in the
-        view registry.
-
-        This should use methods on the object itself and the version object
-        obtained by get_viewable() to render the object to HTML.
-        """
-
-    def is_deletable():
-        """Returns True if object is deletable right now.
-        """
-
-
-class IPublishable(ISilvaObject):
-    """Silva Objects that can be published to the public, and appear
-    in table of contents.
+    They can appear in table of contents.
     """
 
-    # ACCESSORS
+    def is_default():
+        """True if this content object is the default content object
+        of the folder.
+        """
+
     def is_published():
         """Return true if this object is visible to the public.
         """
@@ -118,76 +209,25 @@ class IPublishable(ISilvaObject):
         """
 
 
+class INonPublishable(ISilvaObject):
+    """A content which is not publicly directly viewable
+
+    A non publisable content never appears in the navigation or in
+    table of contents.
+    """
+
+
 ###############################################################
 ### Container
 ###############################################################
 
-class IContainer(IPublishable):
-    """Silva containers.
+class IContainer(IPublishable, IQuotaContainer):
+    """Container
+
+    Contains content.
     """
 
-    used_space = interface.Attribute(u"Used space by assets.")
-
-    # MANIPULATORS
-    def move_object_up(id):
-        """Move object with ``id`` up in the list of ordered publishables.
-        Return true in case of success.
-        """
-
-    def move_object_down(id):
-        """Move object with ``id`` down in the list of ordered publishables.
-        Return true in case of success.
-        """
-
-    def move_to(move_ids, index):
-        """Move ``ids`` just before index.
-        Return true in case success.
-        """
-
-    def action_rename(orig_id, new_id):
-        """Rename subobject with orig_id into new_id.
-        Cannot rename approved or published content.
-        """
-
-    def action_delete(ids):
-        """Delete ids in this container.
-        Cannot delete approved or published content.
-        """
-
-    def action_cut(ids, REQUEST):
-        """Cut ``ids`` in this folder, putting them on clipboard in
-        ``REQUEST``.  Cannot cut approved or published content.
-        """
-
-    def action_copy(ids, REQUEST):
-        """Copy ``ids`` in this folder, putting them on clipboard in
-        ``REQUEST``.
-        """
-
-    def action_paste(REQUEST):
-        """Paste clipboard to this folder.  After paste, approved or
-        published content is automatically unapproved and/or closed.
-        """
-
-    def update_quota(delta):
-        """Update used space with ``delta``, and verify quota for this folder.
-        """
-
     # ACCESSORS
-    def get_silva_addables():
-        """Get a list of meta_type dictionnaries (from
-        ``filtered_meta_types()``) that are addable to this container.
-        """
-
-    def get_silva_addables_all():
-        """Get a list of meta_types of all addables that exist in
-        Silva.
-        """
-
-    def get_silva_addables_allowed():
-        """Gives a list of all meta_types that are explicitly allowed here.
-        """
-
     def get_container():
         """Get the nearest container in the acquisition hierarchy.
         (this one)
@@ -197,65 +237,41 @@ class IContainer(IPublishable):
         """Show this subtree in ``get_tree()``.
         """
 
-    def is_delete_allowed(id):
-        """Return true if subobject with name 'id' can be deleted.
-        This is only allowed if the subobject is not published or
-        approved.
-        """
-
     def get_default():
         """Get the default content object of the folder. If
         no default is available, return None.
         """
 
-    def get_ordered_publishables():
+
+class IOrderableContainer(IContainer):
+    """Ordered Container
+
+    Content in that container can be ordered.
+    """
+
+
+class IFolder(IOrderableContainer):
+    """Folder
+    """
+
+    # Get content
+    def get_ordered_publishables(interface=IPublishable):
         """Get list of active publishables of this folder, in
         order.
+
+        You can restrict futher the list by providing an interface.
         """
 
-    def get_assets():
-        """Get a list of assets objects in this folder.  (not in any
-        fixed order).
-        """
-
-    def get_non_publishables():
+    def get_non_publishables(interface=INonPublishable):
         """Get a list of non-publishable objects in this folder,
         sorted in alphabetical order. This includes assets,
         configuration objects and anything else that is not a
         publishable.
+
+        You can restrict futher the list by providing an interface.
         """
 
-    def get_tree():
-        """Get flattened tree of all active publishables.
-        This is a list of indent, object tuples.
-        """
-
-    def get_container_tree():
-        """Get flattened tree of all sub-containers.
-        This is a list of indent, object tuples.
-        """
-
-    def get_public_tree():
-        """Get tree of all publishables that are public.
-        and not hidden from tocs
-        """
-
-    def get_public_tree_all():
-        """Get tree of all publishables that are public,
-        and not hidden from tocs
-        including the publishables in subpublications.
-        """
-
-    def get_status_tree():
-        """Get tree of all active content objects. For containers,
-        show the default object if available.
-        """
-
-
-class IFolder(IContainer):
-    """Silva Folder
-    """
-
+    # Set addables
     def set_silva_addables_allowed_in_container(addables):
         """Set the list of addables explicitly allowed in this
         container.  If 'addables' is set to None the list is acquired
@@ -265,7 +281,7 @@ class IFolder(IContainer):
 
     def get_silva_addables_allowed_in_container():
         """Get a list of all addables explicitly allowed in this
-        container (and its subcontainers).
+        container.
         """
 
     def is_silva_addables_acquired():
@@ -276,12 +292,16 @@ class IFolder(IContainer):
 
 
 class IPublication(IContainer):
-    """Silva Publication
+    """Publication
     """
 
+    def get_publication():
+        """Return the nearest publication by acquisition.
+        """
+
 
 class IRoot(IPublication):
-    """Silva Root
+    """Root
     """
 
     def get_root():
@@ -303,10 +323,12 @@ class IRoot(IPublication):
         """Returns true if meta_type should not show up in the SMI.
         """
 
+
 ###############################################################
 ### Content
 ###############################################################
 
+
 class IContent(IPublishable):
     """Silva non-container content
 
@@ -315,36 +337,26 @@ class IContent(IPublishable):
     """
     # ACCESSORS
     def get_content():
-        """Used by acquisition to get the nearest containing content object.
-        """
-
-    def is_cacheable():
-        """Return true if the public view of this object can be safely
-        cached.  This means the public view should not contain any
-        dynamically generated content.
-        """
-
-    def is_default():
-        """True if this content object is the default content object of
-        the folder.
+        """Used by acquisition to get the nearest containing content
+        object.
         """
 
 
 class IAutoTOC(IContent):
-    """Auto TOC content
+    """Automatic table of content
     """
 
 
 class IIndexer(IContent):
-    """Indexer content
+    """Indexer
     """
 
-    def getIndexNames():
+    def get_index_names():
         """Returns a list of all index entry names in the index,
         sorted alphabetically.
         """
 
-    def getIndexEntry(indexTitle):
+    def get_index_entry(entry):
         """Returns a list of (title, path) tuples for an entry name in the
         index, sorted alphabetically on title
         """
@@ -366,10 +378,8 @@ class IVersioning(interface.Interface):
     """
 
     # MANIPULATORS
-    def create_version(version_id,
-                       publication_datetime,
-                       expiration_datetime):
-        """Add unapproved version.
+    def create_copy(version_id=None):
+        """Copy a version as currently editable one.
         """
 
     def approve_version():
@@ -428,20 +438,19 @@ class IVersioning(interface.Interface):
 
     # ACCESSORS
 
-    def is_version_approved():
+    def is_approved():
         """Check whether there exists an approved version.
         """
 
-    def is_version_published():
+    def is_published():
         """Check whether there exists a published version.
         """
 
-    def is_version_approval_requested():
+    def is_approval_requested():
         """Check if there exists an unapproved version
         which has a request for approval.
         """
 
-
     def get_unapproved_version():
         """Get the id of the unapproved version.
         """
@@ -485,11 +494,6 @@ class IVersioning(interface.Interface):
         if there is no expiration datetime.
         """
 
-    def get_next_version_status():
-        """Get the status of the next version.
-        Result can be 'not_approved', 'approved', or 'no_next_version'.
-        """
-
     def get_public_version():
         """Get the id of the public version.
         """
@@ -508,11 +512,6 @@ class IVersioning(interface.Interface):
         None if this version never expires.
         """
 
-    def get_public_version_status():
-        """Get the status of the published version.  Result can be
-        ``published``, ``closed``, or ``no_public_version``.
-        """
-
     def get_previous_versions():
         """Get a list of the ids of the most recent versions (that
         are not public anymore. Index 0 is oldest, up is more recent
@@ -524,31 +523,18 @@ class IVersioning(interface.Interface):
         no such version.
         """
 
-    def get_approval_requester():
-        """Return the id of the user requesting approval
-        of the currently unapproved version.
-        XXX fishy: If the request for approval is withdrawn/rejected,
-        this returns the user id of the one having
-        withdrawn/rejected the request.
-        (Maybe write another method for this?)
-        """
 
-    def get_approval_request_message():
-        """Get the current message associated with
-        request for approval; i.e. argument passed the
-        on the last call to "set_approval_request_message".
-        May return None if there is no such message or
-        the message has been purged by an approval.
-        """
-
-    def get_approval_request_datetime():
-        """Get the date when the currently unapproved version
-        did get a request for approval as a DateTime object,
-        or None if there is no such version or request.
-        """
+class IVersionedObject(IVersioning, ISilvaObject, IViewableObject):
+    """Object that can have versions
+    """
 
 
-class IVersionedContent(IVersioning, IContent):
+class IVersionedNonPublishable(IVersionedObject, INonPublishable):
+    """Non publishable object that can have versions
+    """
+
+
+class IVersionedContent(IVersionedObject, IContent):
     """Content that can have versions
     """
 
@@ -561,42 +547,19 @@ class IVersionedContent(IVersioning, IContent):
         """
 
 
-class ICatalogedVersionedContent(IVersionedContent):
-    """Cataloged versioned content
-    """
-
-
-class IVersion(ITitledObject, IAttributeAnnotatable):
+class IVersion(IReferable, ISecurity, ITitledObject, IAttributeAnnotatable):
     """Version of a versioned content
 
     A version effectively store the data of the content.
     """
 
-    def version_status():
-        """Returns the current status of this version (unapproved, approved,
-        public, last closed or closed).
-        """
 
-    def publication_datetime():
-        """Returns the version's publication datetime.
-        """
-
-    def expiration_datetime():
-        """Returns the version's expiration datetime.
-        """
-
-
-class ICatalogedVersion(IVersion):
-    """Cataloged version content
-    """
-
-
-class ILinkVersion(ICatalogedVersion):
+class ILinkVersion(IVersion):
     """Version of a Silva Link content
     """
 
 
-class ILink(ICatalogedVersionedContent):
+class ILink(IVersionedContent):
     """Silva Link content
 
     Silva Link can be used to create links to either other contents in
@@ -608,31 +571,13 @@ class ILink(ICatalogedVersionedContent):
 ###############################################################
 
 
-class INonPublishable(ISilvaObject):
-    """A content which is not publicly viewable
-
-    A non publisable content never appears in the navigation or in
-    table of contents.
-    """
-
-
-class IAsset(INonPublishable):
+class IAsset(INonPublishable, IViewableObject, IQuotaObject):
     """Asset content
 
     An asset is a resource which is not a content object by itself,
     but contain data. Files and Images are assets for example.
     """
 
-    # MANIPULATORS
-
-    def update_quota():
-        """Update used space, and verify quota for this asset.
-        """
-
-    def reset_quota():
-        """Reset the status of the quota.
-        """
-
     # ACCESSORS
 
     def get_filename():
@@ -653,17 +598,30 @@ class IAsset(INonPublishable):
         """
 
 
-class IFile(IAsset, IDirectlyRendered):
+class IDownloableAsset(IAsset):
+
+    def get_html_tag(preview=False, request=None, **extra_attributes):
+        """Return an HTML tag that points to the asset.
+        """
+
+    def get_download_url(preview=False, request=None):
+        """Return an URL to download the asset.
+        """
+
+
+class IFile(IDownloableAsset, IDirectlyRendered):
     """Silva File content to encapsulate "downloadable" data
     """
     # MANIPULATORS
 
-    def set_file_data(file):
+    def set_file(stream, content_type=None, content_encoding=None):
         """Re-upload data for this file object. It will change the
-        ``content_type``, however id, title, etc. will not change.
+        ``content_type`` and ``content_encoding``, however id, title,
+        etc. will not change. If ``content_type`` is None, it will be
+        detected with ``content_encoding``, if possible.
         """
 
-    def set_text_file_data(datastr):
+    def set_text(text):
         """Set content of the file from the given string.
         """
 
@@ -678,57 +636,54 @@ class IFile(IAsset, IDirectlyRendered):
         """
 
     def set_content_encoding(content_encoding):
-        """Set file content encoding.
+        """Set the file content encoding: gzip, bzip2 or none.
         """
 
     # ACCESSORS
 
-    def tag(**kw):
-        """Generate a tag to download file content.
-        """
-
-    def content_type():
+    def get_content_type():
         """Return the file content type as it is send to a visitor
         while downloading the file.
         """
 
-    def content_encoding():
-        """Return the file content encoding as it is send to a visitor
-        while downloading the file.
+    def get_content_encoding():
+        """Return the file content encoding.
         """
 
-    def get_text_content():
+    def get_text():
         """Return the text content of the file or TypesError is it's
         not a text file.
         """
 
-    def get_content():
+    def is_text():
+        """Return True if the file content is text based.
+        """
+
+    def get_file():
         """Return the content of the file in any cases.
         """
 
-    def get_content_fd():
+    def get_file_fd():
         """Return a file descriptor to access the file data.
         """
 
-    def get_download_url():
-        """Obtain the public URL the public could use to download this
-        file. Typically it's the URL used in ``tag``.
-        """
-
 
 class IZODBFile(IFile):
-    """A file in ZODB.
+    """A file stored in ZODB
     """
 
+
 class IFileSystemFile(IFile):
-    """A file on the file system.
+    """A file stored on the file system
     """
 
+
 class IBlobFile(IFile):
-    """A file as a blob.
+    """A file stored with the help of a blob
     """
 
-class IImage(IAsset, IDirectlyRendered):
+
+class IImage(IDownloableAsset, IImageIncluable, IDirectlyRendered):
     """Silva Image
 
     A Silva Image can be used in a Silva Document. An image stored
@@ -767,7 +722,7 @@ class IImage(IAsset, IDirectlyRendered):
         of the image.
         """
 
-    def tag(hires=0, thumbnail=0, **extra_attributes):
+    def get_html_tag(preview=False, request=None, hires=False, thumbnail=False,**extra_attributes):
         """Generate a image tag to render either the original version
         (if ``hires`` set to True), or the thumbnail (if ``thumbnail``
         set to True) or the web version (by default).
@@ -776,18 +731,30 @@ class IImage(IAsset, IDirectlyRendered):
         the image tag as HTML attributes.
         """
 
+
+    def get_download_url(preview=False, request=None, hires=False, thumbnail=False):
+        """Return an URL to download the corresponding image. If ``hires``
+        is True, the URL to the original version will be returned and
+        if ``thumbnail`` is True the URL to the thumbnail will be returned
+        instead.
+        """
+
     def get_crop_box(crop=None):
         """Return a tuple describing the current crop box used to
         create the web image. The describe the coordinate of the top
         left and the bottom right corner.
         """
 
+    def get_canonical_web_scale(scale=None):
+        """returns (width, height) of web image
+        """
+
     def get_orientation():
         """Returns the image orientation: ``square``, ``landscape`` or
         ``portrait``.
         """
 
-    def get_dimensions(img=None):
+    def get_dimensions(thumbnail=False, hires=False):
         """Returns the width and height of the original image as a
         tuple.
 
@@ -811,7 +778,7 @@ class IImage(IAsset, IDirectlyRendered):
         the image.
         """
 
-    def get_image(hires=1, webformat=0):
+    def get_image(hires=True, webformat=False):
         """Return image raw data. If ``hires`` is set to True, the
         original image data is returned. If ``webformat`` is set to
         True, the web version image raw data is returned.
@@ -824,18 +791,18 @@ class IImage(IAsset, IDirectlyRendered):
 
 
 class IGhostAware(interface.Interface):
-    """This mark any ghost content.
+    """This mark any ghost content
     """
 
+    def get_haunted():
+        """Return the final target object.
+        """
+
 
 class IGhostManagable(IGhostAware):
     """Interface for ghosts (and ghost folders).
     """
 
-    def get_haunted():
-        """Return the haunted object.
-        """
-
     def set_haunted(content):
         """Set content to be haunted.
         """
@@ -846,15 +813,35 @@ class IGhostManagable(IGhostAware):
         """
 
 
-class IGhostVersion(IGhostManagable, ICatalogedVersion):
-    """Version of a ghost object.
+class IGhostVersion(IGhostManagable, IVersion):
+    """Version of a ghost
     """
 
 
-class IGhost(IGhostAware, ICatalogedVersionedContent):
-    """Marker interface for "normal" ghosts, i.e. Silva.Ghost.Ghost.
+class IGhost(IGhostAware, IVersionedContent):
+    """Ghost content
     """
 
+
 class IGhostFolder(IGhostManagable, IContainer):
-    """Marker interface for Ghost Folders.
+    """Ghost Folder
     """
+
+
+class IGhostAsset(IGhostManagable, IAsset, IDirectlyRendered):
+    """Ghost Asset
+    """
+
+
+class ISilvaNameChooser(INameChooser):
+    """Choose and check name for Silva content
+    """
+
+    def checkName(identifier, content, file=None, interface=None):
+        """Verify the identifier is valid. Raise ContentError if it is
+        not.
+        """
+
+    def chooseName(identifier, content, file=None, interface=None):
+        """Choose an identifier form name and content (both can be none).
+        """
